package javaLearning2.examples;

import java.awt.*;
import javaLearning2.*;
import javaLearning2.awt.*;
import java.util.Vector;

public class ConversionPanel extends JLPanel {
  JLTextField textField;
  Scrollbar slider;
  JLChoice unitChooser;
  int min = 0;
  int max = 10000;
  Converter controller;
  Unit[] units;
  
  JLFrameWork observerJL;
  
  ConversionPanel(Converter myController, String myTitle, Unit[] myUnits, JLFrameWork obs) {
    
    super(obs); 
    observerJL = obs;    
    initFrameWork ();
    
    //Initialize this ConversionPanel to use a GridBagLayout.
    observerJL.waitAtLine (this, 21);
    JLGridBagConstraints c = new JLGridBagConstraints(obs);
    observerJL.waitAtLine (this, 22);
    JLGridBagLayout gridbag = new JLGridBagLayout(obs);
    observerJL.waitAtLine (this, 23);
    setLayout(gridbag);
    
    //Save arguments in instance variables.
    observerJL.waitAtLine (this, 26);
    controller = myController;
    observerJL.waitAtLine (this, 27);
    units = myUnits;
    
    //Set up default layout constraints.
    observerJL.waitAtLine (this, 30);
    c.fill = GridBagConstraints.HORIZONTAL;
    
    //Add the label.  It displays this panel's title, centered.
    observerJL.waitAtLine (this, 33);
    JLLabel label = new JLLabel(myTitle, Label.CENTER, observerJL); 
    observerJL.waitAtLine (this, 34);
    c.gridwidth = GridBagConstraints.REMAINDER; //It ends a row.
    observerJL.waitAtLine (this, 35);
    gridbag.setConstraints(label, c);
    observerJL.waitAtLine (this, 36);
    add(label);
    
    //Add the text field.  It initially displays "0" and needs
    //to be at least 10 columns wide.
    observerJL.waitAtLine (this, 40);
    textField = new JLTextField("0", 10, observerJL); 
    observerJL.waitAtLine (this, 41);
    c.weightx = 1.0;  //This component should use all available horizontal space...
    observerJL.waitAtLine (this, 42);
    c.gridwidth = 1; //The default value.
    observerJL.waitAtLine (this, 43);
    gridbag.setConstraints(textField, c);
    observerJL.waitAtLine (this, 44);
    add(textField);
    
    //Add the pop-up list (Choice).
    observerJL.waitAtLine (this, 47);
    unitChooser = new JLChoice(observerJL); 
    observerJL.waitAtLine (this, 48);
    for (int i = 0; i < units.length; i++) { //Populate it.
    observerJL.waitAtLine (this, 49);
      unitChooser.addItem(units[i].description);
    }
    observerJL.waitAtLine (this, 51);
    c.weightx = 0.0; //The default value.
    observerJL.waitAtLine (this, 52);
    c.gridwidth = GridBagConstraints.REMAINDER; //End a row.
    observerJL.waitAtLine (this, 53);
    gridbag.setConstraints(unitChooser, c);
    observerJL.waitAtLine (this, 54);
    add(unitChooser);
    
    //Add the slider.  It's horizontal, its initial value is 0,
    //a click increments the value by 100 pixels, and it has the
    //minimum and maximum values specified by the instance variables
    //min and max.
    observerJL.waitAtLine (this, 60);
    slider = new JLScrollBar(Scrollbar.HORIZONTAL, 0, 100, min, max, observerJL);
    observerJL.waitAtLine (this, 61);
    c.gridwidth = 1; //The default value.
    observerJL.waitAtLine (this, 62);
    gridbag.setConstraints(slider, c);
    observerJL.waitAtLine (this, 63);
    add(slider);
  }
  
  /** 
   * Returns the multiplier (units/meter) for the currently
   * selected unit of measurement.
   */
  double getMultiplier() {
    observerJL.waitAMomentAtLine (this, 71);
    int i = unitChooser.getSelectedIndex();
    observerJL.waitAMomentAtLine (this, 72);
    return (units[i].multiplier);
  }
  
  /** Draws a box around this panel. */
  public void paint(Graphics g) {
    Dimension d = size();
    g.drawRect(0,0, d.width - 1, d.height - 1);
  }
  
  /**
   * Puts a little breathing space between
   * the panel and its contents, which lets us draw a box
   * in the paint() method.
   * We add more pixels to the right, to work around a
   * Choice bug.
   */
  public Insets insets() {
    return new Insets(5,5,5,8);
  }
  
  /**
   * Gets the current value in the text field.
   * That's guaranteed to be the same as the value
   * in the scroller (subject to rounding, of course).
   */
  double getValue() {
    double f;
    try {
      observerJL.waitAMomentAtLine (this, 100);
      f = Double.valueOf(textField.getText()).doubleValue(); 
    } catch (java.lang.NumberFormatException e) {
      observerJL.waitAMomentAtLine (this, 102);
      f = 0.0;
    }
    observerJL.waitAMomentAtLine (this, 104);
    return f;
  }
  
  /** Respond to user actions on controls. */
  public boolean action(Event e, Object arg) {
    observerJL.waitAMomentAtLine (this, 109);
    if (e.target instanceof TextField) {
      observerJL.waitAMomentAtLine (this, 110);
      setSliderValue(getValue());
      observerJL.waitAMomentAtLine (this, 111);
      controller.convert(this);
      observerJL.waitAMomentAtLine (this, 112);
      return true;
    }
    observerJL.waitAMomentAtLine (this, 114);
    if (e.target instanceof Choice) {
      observerJL.waitAMomentAtLine (this, 115);
      controller.convert(this);
      observerJL.waitAMomentAtLine (this, 116);
      return true;
    }  
    return false;
  }
  
  /** Respond to the slider. */
  public boolean handleEvent (Event e) {   
    if (e.target instanceof Scrollbar) {
      observerJL.waitAMomentAtLine (this, 124);
      textField.setText (String.valueOf(slider.getValue()));
      observerJL.waitAMomentAtLine (this, 125);
      controller.convert(this);
    }  
    return super.handleEvent (e);
  }
  
  /** Set the values in the slider and text field. */
  void setValue(double f) { 
    observerJL.waitAMomentAtLine (this, 131);
    setSliderValue(f); 
    observerJL.waitAMomentAtLine (this, 132);
    textField.setText(String.valueOf(f));
  }
  
  /** Set the slider value. */
  void setSliderValue(double f) {
    observerJL.waitAMomentAtLine (this, 137);
    int sliderValue = (int)f;
    
    observerJL.waitAMomentAtLine (this, 139);
    if (sliderValue > max)
      sliderValue = max;
    observerJL.waitAMomentAtLine (this, 141);
    if (sliderValue < min)
      sliderValue = min;

    observerJL.waitAMomentAtLine (this, 143);
    slider.setValue(sliderValue);
  }


   public void  initFrameWork () {
    Vector sourceCode = new Vector ();   

    sourceCode.addElement ("");
    sourceCode.addElement ("package javaLearning2.examples;"); 
    sourceCode.addElement (""); 
    sourceCode.addElement ("import java.awt.*;"); 
    sourceCode.addElement ("import javaLearning2.*;"); 
    sourceCode.addElement ("import javaLearning2.awt.*;"); 
    sourceCode.addElement (""); 
    sourceCode.addElement ("public class ConversionPanel extends  Panel {"); 
    sourceCode.addElement ("  TextField textField;"); 
    sourceCode.addElement ("  Scrollbar slider;"); 
    sourceCode.addElement ("  Choice unitChooser;"); 
    sourceCode.addElement ("  int min = 0;"); 
    sourceCode.addElement ("  int max = 10000;"); 
    sourceCode.addElement ("  Converter controller;"); 
    sourceCode.addElement ("  Unit[] units;"); 
    sourceCode.addElement ("  ");  
    sourceCode.addElement ("  ConversionPanel(Converter myController, String myTitle, Unit[] myUnits) {"); 
    sourceCode.addElement ("  ");   
    sourceCode.addElement ("    super();");    
    sourceCode.addElement ("    "); 
    sourceCode.addElement ("    //Initialize this ConversionPanel to use a GridBagLayout."); 
    sourceCode.addElement ("    GridBagConstraints c = new GridBagConstraints(obs);"); 
    sourceCode.addElement ("    GridBagLayout gridbag = new GridBagLayout(obs);"); 
    sourceCode.addElement ("    setLayout(gridbag);"); 
    sourceCode.addElement ("");     
    sourceCode.addElement ("    //Save arguments in instance variables."); 
    sourceCode.addElement ("    controller = myController;"); 
    sourceCode.addElement ("    units = myUnits;"); 
    sourceCode.addElement ("    "); 
    sourceCode.addElement ("    //Set up default layout constraints."); 
    sourceCode.addElement ("    c.fill = GridBagConstraints.HORIZONTAL;"); 
    sourceCode.addElement ("    "); 
    sourceCode.addElement ("    //Add the label.  It displays this panel's title, centered."); 
    sourceCode.addElement ("    Label label = new Label(myTitle, Label.CENTER); "); 
    sourceCode.addElement ("    c.gridwidth = GridBagConstraints.REMAINDER; //It ends a row."); 
    sourceCode.addElement ("    gridbag.setConstraints(label, c);"); 
    sourceCode.addElement ("    add(label);"); 
    sourceCode.addElement ("   ");  
    sourceCode.addElement ("    //Add the text field.  It initially displays \"0\" and needs"); 
    sourceCode.addElement ("    //to be at least 10 columns wide."); 
    sourceCode.addElement ("    textField = new TextField(\"0\", 10); "); 
    sourceCode.addElement ("    c.weightx = 1.0;  //This component should use all available horizontal space..."); 
    sourceCode.addElement ("    c.gridwidth = 1; //The default value."); 
    sourceCode.addElement ("    gridbag.setConstraints(textField, c);"); 
    sourceCode.addElement ("    add(textField);"); 
    sourceCode.addElement ("    "); 
    sourceCode.addElement ("    //Add the pop-up list (Choice)."); 
    sourceCode.addElement ("    unitChooser = new Choice(); "); 
    sourceCode.addElement ("    for (int i = 0; i < units.length; i++) { //Populate it."); 
    sourceCode.addElement ("      unitChooser.addItem(units[i].description);"); 
    sourceCode.addElement ("    }"); 
    sourceCode.addElement ("    c.weightx = 0.0; //The default value."); 
    sourceCode.addElement ("    c.gridwidth = GridBagConstraints.REMAINDER; //End a row."); 
    sourceCode.addElement ("    gridbag.setConstraints(unitChooser, c);"); 
    sourceCode.addElement ("    add(unitChooser);"); 
    sourceCode.addElement (" ");    
    sourceCode.addElement ("    //Add the slider.  It's horizontal, its initial value is 0,"); 
    sourceCode.addElement ("    //a click increments the value by 100 pixels, and it has the"); 
    sourceCode.addElement ("    //minimum and maximum values specified by the instance variables"); 
    sourceCode.addElement ("    //min and max."); 
    sourceCode.addElement ("    slider = new ScrollBar(Scrollbar.HORIZONTAL, 0, 100, min, max);"); 
    sourceCode.addElement ("    c.gridwidth = 1; //The default value."); 
    sourceCode.addElement ("    gridbag.setConstraints(slider, c);"); 
    sourceCode.addElement ("    add(slider);"); 
    sourceCode.addElement ("  }"); 
    sourceCode.addElement ("  "); 
    sourceCode.addElement ("  /** "); 
    sourceCode.addElement ("   * Returns the multiplier (units/meter) for the currently"); 
    sourceCode.addElement ("   * selected unit of measurement."); 
    sourceCode.addElement ("   */"); 
    sourceCode.addElement ("  double getMultiplier() {"); 
    sourceCode.addElement ("    int i = unitChooser.getSelectedIndex();"); 
    sourceCode.addElement ("    return (units[i].multiplier);"); 
    sourceCode.addElement ("  }"); 
    sourceCode.addElement (" ");  
    sourceCode.addElement ("  /** Draws a box around this panel. */"); 
    sourceCode.addElement ("  public void paint(Graphics g) {"); 
    sourceCode.addElement ("    Dimension d = size();"); 
    sourceCode.addElement ("    g.drawRect(0,0, d.width - 1, d.height - 1);"); 
    sourceCode.addElement ("  }"); 
    sourceCode.addElement (" ");  
    sourceCode.addElement ("  /**"); 
    sourceCode.addElement ("   * Puts a little breathing space between"); 
    sourceCode.addElement ("   * the panel and its contents, which lets us draw a box"); 
    sourceCode.addElement ("   * in the paint() method."); 
    sourceCode.addElement ("   * We add more pixels to the right, to work around a"); 
    sourceCode.addElement ("   * Choice bug."); 
    sourceCode.addElement ("   */"); 
    sourceCode.addElement ("  public Insets insets() {"); 
    sourceCode.addElement ("    return new Insets(5,5,5,8);"); 
    sourceCode.addElement ("  }"); 
    sourceCode.addElement ("");   
    sourceCode.addElement ("  /**"); 
    sourceCode.addElement ("   * Gets the current value in the text field."); 
    sourceCode.addElement ("   * That's guaranteed to be the same as the value"); 
    sourceCode.addElement ("   * in the scroller (subject to rounding, of course)."); 
    sourceCode.addElement ("   */"); 
    sourceCode.addElement ("  double getValue() {"); 
    sourceCode.addElement ("    double f;"); 
    sourceCode.addElement ("    try {"); 
    sourceCode.addElement ("      f = Double.valueOf(textField.getText()).doubleValue();");  
    sourceCode.addElement ("    } catch (java.lang.NumberFormatException e) {"); 
    sourceCode.addElement ("      f = 0.0;"); 
    sourceCode.addElement ("    }"); 
    sourceCode.addElement ("    return f;"); 
    sourceCode.addElement ("  }"); 
    sourceCode.addElement ("");  
    sourceCode.addElement ("  /** Respond to user actions on controls. */"); 
    sourceCode.addElement ("  public boolean action(Event e, Object arg) {"); 
    sourceCode.addElement ("    if (e.target instanceof TextField) {"); 
    sourceCode.addElement ("      setSliderValue(getValue());"); 
    sourceCode.addElement ("      controller.convert(this);"); 
    sourceCode.addElement ("      return true;"); 
    sourceCode.addElement ("    }"); 
    sourceCode.addElement ("    if (e.target instanceof Choice) {"); 
    sourceCode.addElement ("      controller.convert(this);"); 
    sourceCode.addElement ("      return true;"); 
    sourceCode.addElement ("    }");  
    sourceCode.addElement ("    return false;"); 
    sourceCode.addElement ("  }"); 
    sourceCode.addElement ("");   
    sourceCode.addElement ("  /** Respond to the slider. */"); 
    sourceCode.addElement ("  public boolean handleEvent(Event e) {"); 
    sourceCode.addElement ("    if (e.target instanceof Scrollbar) {"); 
    sourceCode.addElement ("      textField.setText(String.valueOf(slider.getValue()));"); 
    sourceCode.addElement ("      controller.convert(this);"); 
    sourceCode.addElement ("    } "); 
    sourceCode.addElement ("    return super.handleEvent(e);"); 
    sourceCode.addElement ("  }"); 
    sourceCode.addElement ("");   
    sourceCode.addElement ("  /** Set the values in the slider and text field. */"); 
    sourceCode.addElement ("  void setValue(double f) {"); 
    sourceCode.addElement ("    setSliderValue(f);"); 
    sourceCode.addElement ("    textField.setText(String.valueOf(f));"); 
    sourceCode.addElement ("  }"); 
    sourceCode.addElement (" ");  
    sourceCode.addElement ("  /** Set the slider value. */"); 
    sourceCode.addElement ("  void setSliderValue(double f) {"); 
    sourceCode.addElement ("    int sliderValue = (int)f;"); 
    sourceCode.addElement (" ");     
    sourceCode.addElement ("    if (sliderValue > max)"); 
    sourceCode.addElement ("      sliderValue = max;"); 
    sourceCode.addElement ("    if (sliderValue < min)"); 
    sourceCode.addElement ("      sliderValue = min;"); 
    sourceCode.addElement ("    slider.setValue(sliderValue);"); 
    sourceCode.addElement ("  }"); 
    sourceCode.addElement ("}");  
    sourceCode.addElement (""); 

    observerJL.setSourceCode (this, sourceCode, "ConvesionPanel.java");
  }



}
