
/*
 * 1.0 code.
 */

package javaLearning2.examples;

import java.awt.*;
import java.util.*;
import java.applet.Applet;
import javaLearning2.JLFrameWork;
import javaLearning2.awt.*;



public class Converter extends Applet {
  ConversionPanel metricPanel, usaPanel;
  Unit[] metricDistances = new Unit[3];
  Unit[] usaDistances = new Unit[4];

  JLFrameWork observerJL;

  public Converter (JLFrameWork obs) {
    observerJL = obs;
    initFrameWork ();
  }
  
  /** 
   * Create the ConversionPanels (one for metric, another for U.S.).
   * I used "U.S." because although Imperial and U.S. distance
   * measurements are the same, this program could be extended to
   * include volume measurements, which aren't the same.
     */
  
  public void init() {
    //Use a GridLayout with 2 rows, as many columns as necessary,
    //and 5 pixels of padding around all edges of each cell.
    observerJL.waitAtLine (this, 25);
    setLayout(new GridLayout(2,0,5,5));
    
    //Create Unit objects for metric distances, and then 
    //instantiate a ConversionPanel with these Units.
    observerJL.waitAtLine (this, 28);
    metricDistances[0] = new Unit("Centimeters", 0.01, observerJL);
    observerJL.waitAtLine (this, 29);
    metricDistances[1] = new Unit("Meters", 1.0, observerJL);
    observerJL.waitAtLine (this, 30);
    metricDistances[2] = new Unit("Kilometers", 1000.0, observerJL);
    observerJL.waitAtLine (this, 31);
    metricPanel = new ConversionPanel(this, "Metric System", metricDistances, observerJL);
    
    //Create Unit objects for U.S. distances, and then 
    //instantiate a ConversionPanel with these Units.
    observerJL.waitAtLine (this, 35);
    usaDistances[0] = new Unit("Inches", 0.0254, observerJL);
    observerJL.waitAtLine (this, 36);
    usaDistances[1] = new Unit("Feet", 0.305, observerJL);
    observerJL.waitAtLine (this, 37);
    usaDistances[2] = new Unit("Yards", 0.914, observerJL);
    observerJL.waitAtLine (this, 38);
    usaDistances[3] = new Unit("Miles", 1613.0, observerJL);
    observerJL.waitAtLine (this, 39);
    usaPanel = new ConversionPanel(this, "U.S. System", usaDistances, observerJL);
    
    //Add both ConversionPanels to the Converter.
    observerJL.waitAtLine (this, 42);
    add(metricPanel);
    observerJL.waitAtLine (this, 43);
    add(usaPanel);
    
    //Calling the validate method here can help applets.
    //It's unnecessary when this program runs as an application.
    observerJL.waitAtLine (this, 47);
    validate();
  }
  
  /**
   * Does the conversion from metric to U.S., or vice versa, and
   * updates the appropriate ConversionPanel. 
   */
  void convert(ConversionPanel from) {
    ConversionPanel to;
    
    observerJL.waitAMomentAtLine (this, 56);
    if (from == metricPanel) {
      observerJL.waitAMomentAtLine (this, 57);
      to = usaPanel;
    }
    else {
      observerJL.waitAMomentAtLine (this, 59);
      to = metricPanel;
    }
    observerJL.waitAMomentAtLine (this, 60);
    double multiplier = from.getMultiplier() / to.getMultiplier();
    observerJL.waitAMomentAtLine (this, 61);
    to.setValue(from.getValue() * multiplier);
  }
  
  /** Draws a box around this panel. */
  public void paint(Graphics g) { 
    Dimension d = size(); 
    g.drawRect(0,0, d.width - 1, d.height - 1);
  }
  
  /**
   * Puts a little breathing space between
   * the panel and its contents, which lets us draw a box
   * in the paint() method.
   */
  public Insets insets() { 
    return new Insets(5,5,5,5);
  }

  
  public void  initFrameWork () {
    Vector sourceCode = new Vector ();
    
    
    sourceCode.addElement ("");
    sourceCode.addElement ("package javaLearning.examples;");
    sourceCode.addElement ("import java.awt.*;");
    sourceCode.addElement ("import java.util.*;");
    sourceCode.addElement ("import java.applet.Applet;");
    sourceCode.addElement ("import javaLearning.JLFrameWork;");
    sourceCode.addElement ("import javaLearning.awt.*;");  
    sourceCode.addElement ("");
    sourceCode.addElement ("");
    sourceCode.addElement ("public class Converter extends Applet {");  
    sourceCode.addElement ("");
    sourceCode.addElement ("  ConversionPanel metricPanel, usaPanel;");
    sourceCode.addElement ("  Unit[] metricDistances = new Unit[3];"); 
    sourceCode.addElement ("  Unit[] usaDistances = new Unit[4];"); 
    sourceCode.addElement ("");  
    sourceCode.addElement ("  /**"); 
    sourceCode.addElement ("    * Create the ConversionPanels (one for metric, another for U.S.)");
    sourceCode.addElement ("    * I used \"U.S.\" because although Imperial and U.S. distance");
    sourceCode.addElement ("    * measurements are the same, this program could be extended to");
    sourceCode.addElement ("    * include volume measurements, which aren't the same.");
    sourceCode.addElement ("    */");  
    sourceCode.addElement ("") ;
    sourceCode.addElement ("  public void init() {"); 
    sourceCode.addElement ("    //Use a GridLayout with 2 rows, as many columns as necessary,"); 
    sourceCode.addElement ("    //and 5 pixels of padding around all edges of each cell."); 
    sourceCode.addElement ("    setLayout(new GridLayout(2,0,5,5));");     
    sourceCode.addElement ("    //Create Unit objects for metric distances, and then "); 
    sourceCode.addElement ("    //instantiate a ConversionPanel with these Units."); 
    sourceCode.addElement ("    metricDistances[0] = new Unit(\"Centimeters\", 0.01);"); 
    sourceCode.addElement ("    metricDistances[1] = new Unit(\"Meters\", 1.0);"); 
    sourceCode.addElement ("    metricDistances[2] = new Unit(\"Kilometers\", 1000.0);"); 
    sourceCode.addElement ("    metricPanel = new ConversionPanel(this, \"Metric System\", metricDistances);");
    sourceCode.addElement ("");
    sourceCode.addElement ("    //Create Unit objects for U.S. distances, and then "); 
    sourceCode.addElement ("    //instantiate a ConversionPanel with these Units."); 
    sourceCode.addElement ("    usaDistances[0] = new Unit(\"Inches\", 0.0254);"); 
    sourceCode.addElement ("    usaDistances[1] = new Unit(\"Feet\", 0.305);"); 
    sourceCode.addElement ("    usaDistances[2] = new Unit(\"Yards\", 0.914);"); 
    sourceCode.addElement ("    usaDistances[3] = new Unit(\"Miles\", 1613.0);"); 
    sourceCode.addElement ("    usaPanel = new ConversionPanel(this, \"U.S. System\", usaDistances);"); 
    sourceCode.addElement (" ");    
    sourceCode.addElement ("    //Add both ConversionPanels to the Converter.");
    sourceCode.addElement ( "    add(metricPanel);"); 
    sourceCode.addElement ( "    add(usaPanel);"); 
    sourceCode.addElement ( " ");    
    sourceCode.addElement ("    //Calling the validate method here can help applets.");
    sourceCode.addElement ("    //It's unnecessary when this program runs as an application.");
    sourceCode.addElement ("    validate();");
    sourceCode.addElement ("  }"); 
    sourceCode.addElement ( "");
    sourceCode.addElement ("  /**");
    sourceCode.addElement ( "   * Does the conversion from metric to U.S., or vice versa, and");
    sourceCode.addElement ("    * updates the appropriate ConversionPanel. ");
    sourceCode.addElement ("    */");
    sourceCode.addElement ( "  void convert(ConversionPanel from) {");
    sourceCode.addElement ("     ConversionPanel to;");  
    sourceCode.addElement ("     if (from == metricPanel)");
    sourceCode.addElement ("       to = usaPanel;");
    sourceCode.addElement ("     else");
    sourceCode.addElement ("       to = metricPanel;");
    sourceCode.addElement ("     double multiplier = from.getMultiplier() / to.getMultiplier();");
    sourceCode.addElement ("     to.setValue(from.getValue() * multiplier);");
    sourceCode.addElement ( " ");
    sourceCode.addElement ("");
    sourceCode.addElement ( "    /** Draws a box around this panel. */");
    sourceCode.addElement ( "    public void paint(Graphics g) {");
    sourceCode.addElement ("     Dimension d = size();");
    sourceCode.addElement ("     g.drawRect(0,0, d.width - 1, d.height - 1);");
    sourceCode.addElement ("   }");
    sourceCode.addElement ("");
    sourceCode.addElement ("  /**");
    sourceCode.addElement ("   * Puts a little breathing space between ");  
    sourceCode.addElement ("   * the panel and its contents, which lets us draw a box ");  
    sourceCode.addElement ("   * in the paint() method.  ");
    sourceCode.addElement ("   */    ");
    sourceCode.addElement ("");
    sourceCode.addElement (" public Insets insets() {  ");
    sourceCode.addElement ("   return new Insets(5,5,5,5); ");
    sourceCode.addElement ("  }");
    sourceCode.addElement ("}");
    sourceCode.addElement ("");

    observerJL.setSourceCode (this, sourceCode, "Converter.java");
  }
}



/** Provides a window if this program is run as an application. */
class MainFrame extends Frame {
  MainFrame(String title) {
    super(title);
  } 
  
  public boolean handleEvent(Event e) {
    if (e.id == Event.WINDOW_DESTROY) {
      System.exit(0);
    }
    return super.handleEvent(e);
  }
  

}
